<?php

namespace App\Service;

class DadosVenda {
    
    private $dadosVendaFactory;
    /**
     * Construtor
     *
     * @param $dadosVendaFactory
     */
	public function __construct(\App\Factory\DadosVenda $dadosVendaFactory){
        $this->dadosVendaFactory = $dadosVendaFactory;
    }
    
    public function recuperaDadosVenda($inicio, $fim){
        return $this->dadosVendaFactory->recuperaDadosVenda($inicio, $fim);
    }
    
    /**
     * Recupera os dados das vendas do banco de dados e retorna um Array já formatado para devolver ao client
     *
     * @param String $inicio
     * @param String $fim
     * @return mixed
     */
    public function recuperaDadosVendaGrupo($inicio, $fim){
        return $this->dadosVendaFactory->recuperaVendasPeriodo($inicio, $fim);
    }

    /**
     * Recupera as movimentacões da venda
     *
     * @param String $CDFILIAL
     * @param String $CDCAIXA
     * @param String $NRSEQVENDA
     * @return void
     */
    public function recuperaMovimentacoesVenda($CDFILIAL, $CDCAIXA, $NRSEQVENDA){
        return $this->dadosVendaFactory->recuperaMovimentacoesVenda($CDFILIAL, $CDCAIXA, $NRSEQVENDA);
    }
    
    /**
     * Recupera os grupos de itens da venda
     *
     * @param String $CDFILIAL
     * @param String $CDCAIXA
     * @param String $NRSEQVENDA
     * @return void
     */
    public function recuperaGruposItensVenda($CDFILIAL, $CDCAIXA, $NRSEQVENDA){
        return $this->dadosVendaFactory->recuperaGruposItensVenda($CDFILIAL, $CDCAIXA, $NRSEQVENDA);
    }
    
    /**
     * Recupera os itens do grupo da venda
     *
     * @param String $CDFILIAL
     * @param String $CDCAIXA
     * @param String $NRSEQVENDA
     * @return void
     */
    public function recuperaItensGrupoVenda($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NMSUBGRPROD){
        return $this->dadosVendaFactory->recuperaItensGrupoVenda($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NMSUBGRPROD);
    }
    
    /**
     * Formata o array vindo do banco para um array mais legível ao client
     *
     * @param mixed $dadosVendas
     * @return mixed
     */
    public function formataVendasGrupo($dadosVendas){
        try{
            $retorno = array();
            foreach($dadosVendas as $dadosVenda){
                $dadosVenda['TOTAL'] = str_replace(",", ".", $dadosVenda['TOTAL']);
                $dadosFormatados = array(
                    "dataVenda"     => $dadosVenda['DTENTRVENDA'],
                    "grupoFilial"   => $dadosVenda['GRUPO'],
                    "nomeFilial"    => $dadosVenda['FILIAL'],
                    "caixa"         => $dadosVenda['NMCAIXA'],
                    "tipoVenda"     => $dadosVenda['TIPOVENDA'],
                    "operador"      => $dadosVenda['NMOPERADOR'],
                    "nrVenda"       => $dadosVenda['NRSEQVENDA'],
                    "valorTotal"    => bcdiv($dadosVenda['TOTAL'], 1, 2),
                    "qtdePessoas"   => $dadosVenda['PESSOAS'],
                    "movimentacoes" => array(),
                    "grupos"        => array()
                );
                foreach($dadosVenda['MOVIMENTACOES'] as $movimentacao){
                    $movimentacao['VRMOVIVEND'] = str_replace(",", ".", $movimentacao['VRMOVIVEND']);
                    $data = array(
                        'tipo'    => $movimentacao['IDTIPOMOVIVE'],
                        'valor'   => bcdiv($movimentacao['VRMOVIVEND'], 1, 2),
                        'codigo'  => $movimentacao['CDTIPORECE'],
                        'nome'    => $movimentacao['NMTIPORECE']
                    );
                    array_push($dadosFormatados['movimentacoes'], $data);
                }
                $i = 0;
                foreach($dadosVenda['GRUPOS'] as $grupo){
                    $data = array(
                        'nome' => $grupo['NMSUBGRPROD']
                    );
                    array_push($dadosFormatados['grupos'], $data);
                    
                    $dadosFormatados['grupos'][$i]['itens'] = array();
                    foreach($grupo['ITENS'] as $item){
                        $data = array(
                            'produto' => $item['CDPRODUTO'],
                            'nome'    => $item['NMPRODUTO']
                        );
                        array_push($dadosFormatados['grupos'][$i]['itens'], $data);
                    }
                    $i++;
                }
                array_push($retorno, $dadosFormatados);
            }
            return $retorno;
        }catch(\Exception $e){
            throw new \Exception("Ocorreu um erro ao recuperar os dados da venda.");
        }
        
    }
    
    public function formataRelatorio($turnos){
        $relatorio = array();
        foreach($turnos as $turno){
            $totalReceb = 0;
            $totalSangria = 0;
            $turnoData = array(
                'filial' => $turno['FILIAL'],
                'operador' => $turno['OPERADOR'],
                'abertura_caixa' => $turno['DTABERCAIX'],
                'fechamento_caixa' => $turno['DTFECHCAIX'],
                'caixa' => $turno['CDCAIXA'],
                'total_recebimentos' => 0,
                'total_sangria' => 0,
                'total_diferenca' => 0,
                'pagamentos' => array(),
                'vendas' => array()
            );
            $pagamentos = array();
            $movimentacoes = $turno['MOVIMENTACOES'];
            foreach($movimentacoes as $movimentacao){
                $movimentacao['SOMA'] = str_replace(",", ".", $movimentacao['SOMA']);
                $cdtiporece = $movimentacao['CDTIPORECE'];
                if($movimentacao['IDTIPOMOVIVE'] == 'G'){
                    $totalSangria = bcadd($totalSangria, $movimentacao['SOMA'], 2);
                    $pagamentos[$cdtiporece]['sangria'] = bcadd($movimentacao['SOMA'], 0, 2);
                    $pagamentos[$cdtiporece]['diferenca'] = bcsub($pagamentos[$cdtiporece]['valor_sistema'], $pagamentos[$cdtiporece]['sangria'], 2);
                }else if(isset($pagamentos[$cdtiporece])){
                    if($movimentacao['IDTIPOMOVIVE'] == 'S'){
                        $pagamentos[$cdtiporece]['valor_sistema'] = bcsub($pagamentos[$cdtiporece]['valor_sistema'], $movimentacao['SOMA'], 2);
                    }else if($movimentacao['IDTIPOMOVIVE'] == 'E'){
                        $totalReceb = bcadd($totalReceb, $movimentacao['SOMA'], 2);
                        $pagamentos[$cdtiporece]['valor_sistema'] = bcadd($pagamentos[$cdtiporece]['valor_sistema'], $movimentacao['SOMA'], 2);
                    }
                }else{
                    $pagamentos[$cdtiporece]['codigo'] = $movimentacao['CDTIPORECE'];
                    $pagamentos[$cdtiporece]['nome'] = $movimentacao['NMTIPORECE'];
                    $pagamentos[$cdtiporece]['valor_sistema'] = bcadd($movimentacao['SOMA'], 0, 2);
                    $pagamentos[$cdtiporece]['sangria'] = 0;
                    $pagamentos[$cdtiporece]['diferenca'] = 0;
                    $totalReceb = bcadd($totalReceb, $movimentacao['SOMA'], 2);
                }
                
                $pagamentos[$cdtiporece]['valor_sistema'] = floatval($pagamentos[$cdtiporece]['valor_sistema']);
                $pagamentos[$cdtiporece]['sangria'] = floatval($pagamentos[$cdtiporece]['sangria']);
                $pagamentos[$cdtiporece]['diferenca'] = floatval($pagamentos[$cdtiporece]['diferenca']);
                
            }
            $pagamentosData = array_values($pagamentos);
            $turnoData['pagamentos'] = $pagamentosData;
            $turnoData['total_recebimentos'] = floatval($totalReceb);
            $turnoData['total_sangria'] = floatval($totalSangria);
            $turnoData['total_diferenca'] = floatval(bcsub($totalReceb, $totalSangria, 2));
            
            $vendas = array();
            $informacaoVendas = $turno['VENDAS'];
            foreach($informacaoVendas as $venda){
                $vendaData = array(
                    "tipoVenda"     => $venda['TIPOVENDA'],
                    "valorTotal"    => floatval(bcdiv(str_replace(",",".", $venda['VRTOTVENDA']), 1, 2)),
                    "qtdePessoas"   => floatval($venda['PESSOAS']),
                    "qtdeVendas"    => floatval($venda['QUANTVENDAS'])
                );
                array_push($vendas, $vendaData);
            }
            $turnoData['vendas'] = $vendas;
            
            array_push($relatorio, $turnoData);
        }
        return $relatorio;
    }
    
    public function recuperarTurnos($inicio, $fim, $filiais){
        return $this->dadosVendaFactory->recuperarTurnos($inicio, $fim, $filiais);
    }
    
    public function recuperarPagamentosTurno($turnos){
        
        return $turnos;
    }
    
    public function recuperaRelatorio($inicio, $fim, $filiais){
        $turnos = $this->recuperarTurnos($inicio, $fim, $filiais);
        foreach($turnos as &$turno){
            $cdfilial = $turno['CDFILIAL'];
            $cdcaixa = $turno['CDCAIXA'];
            $dtinicio = $turno['DTABERCAIX'];
            $dtfim = $turno['DTFECHCAIX'];
            $movimentacoes = $this->dadosVendaFactory->recuperaPagamentos($cdfilial, $cdcaixa, $dtinicio, $dtfim);
            $vendas = $this->dadosVendaFactory->recuperaVendasTurno($cdfilial, $cdcaixa, $dtinicio, $dtfim);
            $turno['MOVIMENTACOES'] = $movimentacoes;
            $turno['VENDAS'] = $vendas;
        }
        unset($turno);
        return $turnos;
    }
    
    public function formataDadosVenda($dadosVendas){
        try{
            $retorno = array();
            foreach($dadosVendas as $dadosVenda){
                $dadosFormatados = array(
                    "dataVenda"   => $dadosVenda['DTENTRVENDA'],
                    "grupoFilial" => $dadosVenda['GRUPO'],
                    "nomeFilial"  => $dadosVenda['FILIAL'],
                    "tipoVenda"   => $dadosVenda['TIPOVENDA'],
                    "valorTotal"  => $dadosVenda['TOTAL'],
                    "qtdePessoas" => $dadosVenda['PESSOAS']    
                );
                array_push($retorno, $dadosFormatados);
            }
            return $retorno;
        }catch(\Exception $e){
            throw new \Exception("Ocorreu um erro ao recuperar os dados da venda.");
        }
        
    }
    
}

